require 'spec_helper'

module StartHer
  describe Heartbeat do
    include Utils

    let(:uuid) { 'bb8dd4f2-5e3c-420e-9b18-535a335052a2' }

    before do
      allow(SecureRandom).to receive(:uuid).and_return(uuid)
    end

    let(:request) { { 'data' => { service: 'ping_service' } } }
    let(:block) { ->(response) { response[:custom_key] = 'custom' } }
    subject { Heartbeat.new(request, StartHer, &block) }
    let(:raw_response) do
      {
        id: subject.msid(StartHer),
        generated_at: Time.now.utc.to_s,
        service_name: 'StartHer',
        version: StartHer::VERSION,
        ping: request['data']
      }
    end
    let(:response) do
      {
        id: uuid,
        generated_at: Time.now.utc.to_s,
        data: raw_response
      }
    end
    let(:raw_response_2) { raw_response.merge(custom_key: 'custom') }
    let(:response_2) { response.merge(data: raw_response_2) }

    describe '.call' do
      it 'publishes the pong message' do
        expect_any_instance_of(Heartbeat).to receive(:publish)
          .with('hb_pong', response_2)
        Heartbeat.call(request, StartHer, &block)
      end
    end

    describe '#initialize' do
      it 'formats response' do
        expect(subject.response).to eq(raw_response)
      end
    end

    describe '#call' do
      it 'publishes the pong message' do
        expect(subject).to receive(:publish)
          .with('hb_pong', response_2)
        subject.call
      end
    end
  end
end
