require 'spec_helper'

module StartHer
  describe Publisher do
    class TestPublisher
      include Publisher
    end
    subject { TestPublisher.new }
    let(:channel) { 'channel' }
    let(:message) { 'message' }

    describe '#publish' do
      using StartHer::Refinements

      it 'publishes the message' do
        expect_any_instance_of(StubedRedis).to receive(:publish)
          .with(channel, message.to_msgpack)

        subject.publish(channel, message)
      end
    end

    describe '#reliable_publish' do
      using StartHer::Refinements

      it 'backlogs the message' do
        expect_any_instance_of(StubedRedis).to receive(:lpush)
          .with(channel.to_current_backlog, message.to_msgpack)

        subject.reliable_publish(channel, message)
      end

      it 'adds TTL on backlog' do
        subject.reliable_publish(channel, message)

        expect(subject.send(:client).ttl(channel.to_current_backlog)).to eq(604_800)
      end

      it 'publishes the message' do
        expect_any_instance_of(StubedRedis).to receive(:publish)
          .with(channel, message.to_msgpack)

        subject.reliable_publish(channel, message)
      end
    end
  end
end
