require 'spec_helper'

module StartHer
  describe RetryPolicies do
    describe '#exponential_backoff' do
      let(:dummy_class) { Class.new.extend(RetryPolicies) }
      let(:options) { { elapsed_time_secs: 1, multiplier: 2 } }

      it 'updates time with defined policy' do
        retries_time = []
        started_at = Time.now.utc
        CustomHelpers.suppress(RuntimeError) do
          dummy_class.exponential_backoff(options, RuntimeError) do
            retries_time << CustomHelpers.truncate_float(Time.now.utc - started_at, 1)
            fail RuntimeError.new('The error')
          end
        end
        expect(retries_time).to contain_exactly(0.0, 0.2, 0.6, 1.0)
      end

      context 'when retry policy ends' do
        let(:options) { { elapsed_time_secs: 0.4, multiplier: 2 } }
        let(:exponential_backoff) do
          lambda do |options|
            dummy_class.exponential_backoff(options, RuntimeError) do
              fail RuntimeError.new('The tested error')
            end
          end
        end

        it 'stops at elapsed time' do
          started_at = Time.now.utc
          CustomHelpers.suppress(RuntimeError) do
            exponential_backoff.call(options)
          end
          expect((Time.now.utc - started_at).round(1)).to eq(options[:elapsed_time_secs])
        end

        it 'raises an error' do
          expect { exponential_backoff.call(options) }.to raise_error(RuntimeError)
        end
      end
    end
  end
end
