require 'spec_helper'

module StartHer
  describe Subscriber do
    let(:channels) { ['channel.1', 'channel.2'] }

    class TestSubscriber
      include Subscriber

      subscriber_options channels: ['channel.*']

      def process_message(_channel, _message)
      end
    end
    subject { TestSubscriber.new(['channel.*']) }

    describe '.run!' do
      it 'subscribes to channels' do
        expect_any_instance_of(Subscriber).to receive(:psubscribe).with(['channel.*'])
        TestSubscriber.run!
      end
    end

    describe '#resync_message' do
      let(:message) { 'my message' }
      let(:publisher) do
        ->(chan, msg) { Object.new.extend(StartHer::Publisher).reliable_publish(chan, msg) }
      end

      before do
        publisher.call(channels.first, message)
        publisher.call(channels.last, message)
      end

      it 'resynchronizes messages' do
        expect(subject).to receive(:process_message)
          .with(channels.first, message).ordered
        expect(subject).to receive(:process_message)
          .with(channels.last, message).ordered

        subject.run!
      end
    end

    describe '#psubscribe' do
      context 'when it receives a heartbeat' do
        let(:ping) do
          {
            'id' => Object.new.extend(StartHer::Utils).msid,
            'timestamp' => Time.now.utc.to_s,
            'service' => 'KeepHer'
          }
        end
        let(:publisher) do
          ->(chan, msg) { Object.new.extend(StartHer::Publisher).publish(chan, msg) }
        end

        before do
          subject.run!
        end

        it 'calls the heartbeat responder' do
          expect(Heartbeat).to receive(:call)
            .with(ping, 'StartHer', heartbeat: subject.options[:heartbeat])

          publisher.call(subject.options[:heartbeat][:in], ping)
        end
      end
    end
  end
end
